package de.unikoblenz.fgbks.base.utils.boundary;

import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheck;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType;
import de.unikoblenz.fgbks.base.utils.boundary.impl.InvalidBoundary;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import java.util.HashMap;
import org.apache.commons.lang3.Validate;

public abstract class AbstractBoundary<T extends Comparable<? super T>> implements Boundary {

  private HashMap<BoundaryCheckType, BoundaryCheck> checkFunctions;
  private Class<T> type;
  private String text;

  public AbstractBoundary(String text, Class<T> type) {
    this.text = text;
    this.type = type;
    parse(text);
    validate();
    checkFunctions = new HashMap<>(getChecker());
  }

  @Override
  public String getText() {
    return text;
  }

  @Override
  public VTypeRef getTypeRef() {
    return VTypeRef.getTypeRefFromJavaClass(type);
  }

  @Override
  public boolean checkWith(BoundaryCheckType checkType, Boundary other) {
    if (other instanceof InvalidBoundary) {
      return false;
    }
    BoundaryCheck checkFunction = getTestFunction(checkType);
    if (checkFunction == null || !isValidOperation(checkType, other)) {
      throw new BoundaryOperationNotSupportedException(); // TODO: add error text
    }
    return checkFunction.check(this, other);
  }

  public boolean isValidOperation(BoundaryCheckType checkType, Boundary other) {
    return this.getTypeRef() == other.getTypeRef() && isValidOperation(checkType);
  }

  public boolean isValidOperation(BoundaryCheckType checkType) {
    return checkFunctions.containsKey(Validate.notNull(checkType));
  }

  protected BoundaryCheck getTestFunction(BoundaryCheckType checkType) {
    return checkFunctions.get(checkType);
  }

  protected abstract void parse(String text);

  protected abstract void validate();

  protected abstract HashMap<BoundaryCheckType, BoundaryCheck> getChecker();

  protected abstract String getParsedText();

  @Override
  public String toString() {
    return "{" + getText() + "},{" + getParsedText() + "}";
  }
}
